# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/IR.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gtirb.proto import AuxData_pb2 as gtirb_dot_proto_dot_AuxData__pb2
from gtirb.proto import CFG_pb2 as gtirb_dot_proto_dot_CFG__pb2
from gtirb.proto import Module_pb2 as gtirb_dot_proto_dot_Module__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/IR.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_pb=_b('\n\x14gtirb/proto/IR.proto\x12\x0bgtirb.proto\x1a\x19gtirb/proto/AuxData.proto\x1a\x15gtirb/proto/CFG.proto\x1a\x18gtirb/proto/Module.proto\"\x96\x02\n\x02IR\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12$\n\x07modules\x18\x03 \x03(\x0b\x32\x13.gtirb.proto.Module\x12.\n\x08\x61ux_data\x18\x05 \x03(\x0b\x32\x1c.gtirb.proto.IR.AuxDataEntry\x12\x0f\n\x07version\x18\x06 \x01(\r\x12\x1d\n\x03\x63\x66g\x18\x07 \x01(\x0b\x32\x10.gtirb.proto.CFG\x1a\x44\n\x0c\x41uxDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.gtirb.proto.AuxData:\x02\x38\x01J\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05R\x06tablesR\x0emain_module_idR\x12\x61ux_data_containerB\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
  ,
  dependencies=[gtirb_dot_proto_dot_AuxData__pb2.DESCRIPTOR,gtirb_dot_proto_dot_CFG__pb2.DESCRIPTOR,gtirb_dot_proto_dot_Module__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_IR_AUXDATAENTRY = _descriptor.Descriptor(
  name='AuxDataEntry',
  full_name='gtirb.proto.IR.AuxDataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='gtirb.proto.IR.AuxDataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='gtirb.proto.IR.AuxDataEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=268,
  serialized_end=336,
)

_IR = _descriptor.Descriptor(
  name='IR',
  full_name='gtirb.proto.IR',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='gtirb.proto.IR.uuid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='modules', full_name='gtirb.proto.IR.modules', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aux_data', full_name='gtirb.proto.IR.aux_data', index=2,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='version', full_name='gtirb.proto.IR.version', index=3,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cfg', full_name='gtirb.proto.IR.cfg', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_IR_AUXDATAENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=114,
  serialized_end=392,
)

_IR_AUXDATAENTRY.fields_by_name['value'].message_type = gtirb_dot_proto_dot_AuxData__pb2._AUXDATA
_IR_AUXDATAENTRY.containing_type = _IR
_IR.fields_by_name['modules'].message_type = gtirb_dot_proto_dot_Module__pb2._MODULE
_IR.fields_by_name['aux_data'].message_type = _IR_AUXDATAENTRY
_IR.fields_by_name['cfg'].message_type = gtirb_dot_proto_dot_CFG__pb2._CFG
DESCRIPTOR.message_types_by_name['IR'] = _IR

IR = _reflection.GeneratedProtocolMessageType('IR', (_message.Message,), dict(

  AuxDataEntry = _reflection.GeneratedProtocolMessageType('AuxDataEntry', (_message.Message,), dict(
    DESCRIPTOR = _IR_AUXDATAENTRY,
    __module__ = 'gtirb.proto.IR_pb2'
    # @@protoc_insertion_point(class_scope:gtirb.proto.IR.AuxDataEntry)
    ))
  ,
  DESCRIPTOR = _IR,
  __module__ = 'gtirb.proto.IR_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.IR)
  ))
_sym_db.RegisterMessage(IR)
_sym_db.RegisterMessage(IR.AuxDataEntry)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\032com.grammatech.gtirb.proto'))
_IR_AUXDATAENTRY.has_options = True
_IR_AUXDATAENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
