# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/CodeBlock.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/CodeBlock.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_pb=_b('\n\x1bgtirb/proto/CodeBlock.proto\x12\x0bgtirb.proto\"K\n\tCodeBlock\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x0c\n\x04size\x18\x03 \x01(\x04\x12\x13\n\x0b\x64\x65\x63ode_mode\x18\x04 \x01(\x04J\x04\x08\x02\x10\x03R\x07\x61\x64\x64ressB\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_CODEBLOCK = _descriptor.Descriptor(
  name='CodeBlock',
  full_name='gtirb.proto.CodeBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='gtirb.proto.CodeBlock.uuid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='size', full_name='gtirb.proto.CodeBlock.size', index=1,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='decode_mode', full_name='gtirb.proto.CodeBlock.decode_mode', index=2,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=44,
  serialized_end=119,
)

DESCRIPTOR.message_types_by_name['CodeBlock'] = _CODEBLOCK

CodeBlock = _reflection.GeneratedProtocolMessageType('CodeBlock', (_message.Message,), dict(
  DESCRIPTOR = _CODEBLOCK,
  __module__ = 'gtirb.proto.CodeBlock_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.CodeBlock)
  ))
_sym_db.RegisterMessage(CodeBlock)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\032com.grammatech.gtirb.proto'))
# @@protoc_insertion_point(module_scope)
