# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/SymbolicExpression.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/SymbolicExpression.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_pb=_b('\n$gtirb/proto/SymbolicExpression.proto\x12\x0bgtirb.proto\"4\n\rSymStackConst\x12\x0e\n\x06offset\x18\x01 \x01(\x05\x12\x13\n\x0bsymbol_uuid\x18\x02 \x01(\x0c\"3\n\x0cSymAddrConst\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x13\n\x0bsymbol_uuid\x18\x02 \x01(\x0c\"X\n\x0bSymAddrAddr\x12\r\n\x05scale\x18\x01 \x01(\x03\x12\x0e\n\x06offset\x18\x02 \x01(\x03\x12\x14\n\x0csymbol1_uuid\x18\x03 \x01(\x0c\x12\x14\n\x0csymbol2_uuid\x18\x04 \x01(\x0c\"\xb0\x01\n\x12SymbolicExpression\x12\x31\n\x0bstack_const\x18\x01 \x01(\x0b\x32\x1a.gtirb.proto.SymStackConstH\x00\x12/\n\naddr_const\x18\x02 \x01(\x0b\x32\x19.gtirb.proto.SymAddrConstH\x00\x12-\n\taddr_addr\x18\x03 \x01(\x0b\x32\x18.gtirb.proto.SymAddrAddrH\x00\x42\x07\n\x05valueB\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_SYMSTACKCONST = _descriptor.Descriptor(
  name='SymStackConst',
  full_name='gtirb.proto.SymStackConst',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='gtirb.proto.SymStackConst.offset', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol_uuid', full_name='gtirb.proto.SymStackConst.symbol_uuid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=53,
  serialized_end=105,
)


_SYMADDRCONST = _descriptor.Descriptor(
  name='SymAddrConst',
  full_name='gtirb.proto.SymAddrConst',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='gtirb.proto.SymAddrConst.offset', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol_uuid', full_name='gtirb.proto.SymAddrConst.symbol_uuid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=158,
)


_SYMADDRADDR = _descriptor.Descriptor(
  name='SymAddrAddr',
  full_name='gtirb.proto.SymAddrAddr',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='scale', full_name='gtirb.proto.SymAddrAddr.scale', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offset', full_name='gtirb.proto.SymAddrAddr.offset', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol1_uuid', full_name='gtirb.proto.SymAddrAddr.symbol1_uuid', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol2_uuid', full_name='gtirb.proto.SymAddrAddr.symbol2_uuid', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=160,
  serialized_end=248,
)


_SYMBOLICEXPRESSION = _descriptor.Descriptor(
  name='SymbolicExpression',
  full_name='gtirb.proto.SymbolicExpression',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stack_const', full_name='gtirb.proto.SymbolicExpression.stack_const', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='addr_const', full_name='gtirb.proto.SymbolicExpression.addr_const', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='addr_addr', full_name='gtirb.proto.SymbolicExpression.addr_addr', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='gtirb.proto.SymbolicExpression.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=251,
  serialized_end=427,
)

_SYMBOLICEXPRESSION.fields_by_name['stack_const'].message_type = _SYMSTACKCONST
_SYMBOLICEXPRESSION.fields_by_name['addr_const'].message_type = _SYMADDRCONST
_SYMBOLICEXPRESSION.fields_by_name['addr_addr'].message_type = _SYMADDRADDR
_SYMBOLICEXPRESSION.oneofs_by_name['value'].fields.append(
  _SYMBOLICEXPRESSION.fields_by_name['stack_const'])
_SYMBOLICEXPRESSION.fields_by_name['stack_const'].containing_oneof = _SYMBOLICEXPRESSION.oneofs_by_name['value']
_SYMBOLICEXPRESSION.oneofs_by_name['value'].fields.append(
  _SYMBOLICEXPRESSION.fields_by_name['addr_const'])
_SYMBOLICEXPRESSION.fields_by_name['addr_const'].containing_oneof = _SYMBOLICEXPRESSION.oneofs_by_name['value']
_SYMBOLICEXPRESSION.oneofs_by_name['value'].fields.append(
  _SYMBOLICEXPRESSION.fields_by_name['addr_addr'])
_SYMBOLICEXPRESSION.fields_by_name['addr_addr'].containing_oneof = _SYMBOLICEXPRESSION.oneofs_by_name['value']
DESCRIPTOR.message_types_by_name['SymStackConst'] = _SYMSTACKCONST
DESCRIPTOR.message_types_by_name['SymAddrConst'] = _SYMADDRCONST
DESCRIPTOR.message_types_by_name['SymAddrAddr'] = _SYMADDRADDR
DESCRIPTOR.message_types_by_name['SymbolicExpression'] = _SYMBOLICEXPRESSION

SymStackConst = _reflection.GeneratedProtocolMessageType('SymStackConst', (_message.Message,), dict(
  DESCRIPTOR = _SYMSTACKCONST,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymStackConst)
  ))
_sym_db.RegisterMessage(SymStackConst)

SymAddrConst = _reflection.GeneratedProtocolMessageType('SymAddrConst', (_message.Message,), dict(
  DESCRIPTOR = _SYMADDRCONST,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymAddrConst)
  ))
_sym_db.RegisterMessage(SymAddrConst)

SymAddrAddr = _reflection.GeneratedProtocolMessageType('SymAddrAddr', (_message.Message,), dict(
  DESCRIPTOR = _SYMADDRADDR,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymAddrAddr)
  ))
_sym_db.RegisterMessage(SymAddrAddr)

SymbolicExpression = _reflection.GeneratedProtocolMessageType('SymbolicExpression', (_message.Message,), dict(
  DESCRIPTOR = _SYMBOLICEXPRESSION,
  __module__ = 'gtirb.proto.SymbolicExpression_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.SymbolicExpression)
  ))
_sym_db.RegisterMessage(SymbolicExpression)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\032com.grammatech.gtirb.proto'))
# @@protoc_insertion_point(module_scope)
