# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gtirb/proto/Module.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gtirb.proto import AuxData_pb2 as gtirb_dot_proto_dot_AuxData__pb2
from gtirb.proto import ProxyBlock_pb2 as gtirb_dot_proto_dot_ProxyBlock__pb2
from gtirb.proto import Section_pb2 as gtirb_dot_proto_dot_Section__pb2
from gtirb.proto import Symbol_pb2 as gtirb_dot_proto_dot_Symbol__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gtirb/proto/Module.proto',
  package='gtirb.proto',
  syntax='proto3',
  serialized_pb=_b('\n\x18gtirb/proto/Module.proto\x12\x0bgtirb.proto\x1a\x19gtirb/proto/AuxData.proto\x1a\x1cgtirb/proto/ProxyBlock.proto\x1a\x19gtirb/proto/Section.proto\x1a\x18gtirb/proto/Symbol.proto\"\xa9\x04\n\x06Module\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x13\n\x0b\x62inary_path\x18\x02 \x01(\t\x12\x16\n\x0epreferred_addr\x18\x03 \x01(\x04\x12\x14\n\x0crebase_delta\x18\x04 \x01(\x03\x12,\n\x0b\x66ile_format\x18\x05 \x01(\x0e\x32\x17.gtirb.proto.FileFormat\x12\x1d\n\x03isa\x18\x06 \x01(\x0e\x32\x10.gtirb.proto.ISA\x12\x0c\n\x04name\x18\x07 \x01(\t\x12$\n\x07symbols\x18\t \x03(\x0b\x32\x13.gtirb.proto.Symbol\x12(\n\x07proxies\x18\x10 \x03(\x0b\x32\x17.gtirb.proto.ProxyBlock\x12&\n\x08sections\x18\x0c \x03(\x0b\x32\x14.gtirb.proto.Section\x12\x32\n\x08\x61ux_data\x18\x11 \x03(\x0b\x32 .gtirb.proto.Module.AuxDataEntry\x12\x13\n\x0b\x65ntry_point\x18\x12 \x01(\x0c\x1a\x44\n\x0c\x41uxDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.gtirb.proto.AuxData:\x02\x38\x01J\x04\x08\x08\x10\tJ\x04\x08\n\x10\x0bJ\x04\x08\x0b\x10\x0cJ\x04\x08\r\x10\x0eJ\x04\x08\x0e\x10\x0fJ\x04\x08\x0f\x10\x10R\x0eimage_byte_mapR\x04\x64\x61taR\x11symbolic_operandsR\x12\x61ux_data_containerR\x06\x62locksR\x03\x63\x66g*|\n\nFileFormat\x12\x14\n\x10\x46ormat_Undefined\x10\x00\x12\x08\n\x04\x43OFF\x10\x01\x12\x07\n\x03\x45LF\x10\x02\x12\x06\n\x02PE\x10\x03\x12\x0e\n\nIdaProDb32\x10\x04\x12\x0e\n\nIdaProDb64\x10\x05\x12\t\n\x05XCOFF\x10\x06\x12\t\n\x05MACHO\x10\x07\x12\x07\n\x03RAW\x10\x08*\x86\x01\n\x03ISA\x12\x11\n\rISA_Undefined\x10\x00\x12\x08\n\x04IA32\x10\x01\x12\t\n\x05PPC32\x10\x02\x12\x07\n\x03X64\x10\x03\x12\x07\n\x03\x41RM\x10\x04\x12\x17\n\x13ValidButUnsupported\x10\x05\x12\t\n\x05PPC64\x10\x06\x12\t\n\x05\x41RM64\x10\x07\x12\n\n\x06MIPS32\x10\x08\x12\n\n\x06MIPS64\x10\tB\x1c\n\x1a\x63om.grammatech.gtirb.protob\x06proto3')
  ,
  dependencies=[gtirb_dot_proto_dot_AuxData__pb2.DESCRIPTOR,gtirb_dot_proto_dot_ProxyBlock__pb2.DESCRIPTOR,gtirb_dot_proto_dot_Section__pb2.DESCRIPTOR,gtirb_dot_proto_dot_Symbol__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_FILEFORMAT = _descriptor.EnumDescriptor(
  name='FileFormat',
  full_name='gtirb.proto.FileFormat',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Format_Undefined', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COFF', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ELF', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PE', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IdaProDb32', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IdaProDb64', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='XCOFF', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MACHO', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RAW', index=8, number=8,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=707,
  serialized_end=831,
)
_sym_db.RegisterEnumDescriptor(_FILEFORMAT)

FileFormat = enum_type_wrapper.EnumTypeWrapper(_FILEFORMAT)
_ISA = _descriptor.EnumDescriptor(
  name='ISA',
  full_name='gtirb.proto.ISA',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ISA_Undefined', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IA32', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PPC32', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='X64', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ARM', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ValidButUnsupported', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PPC64', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ARM64', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MIPS32', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MIPS64', index=9, number=9,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=834,
  serialized_end=968,
)
_sym_db.RegisterEnumDescriptor(_ISA)

ISA = enum_type_wrapper.EnumTypeWrapper(_ISA)
Format_Undefined = 0
COFF = 1
ELF = 2
PE = 3
IdaProDb32 = 4
IdaProDb64 = 5
XCOFF = 6
MACHO = 7
RAW = 8
ISA_Undefined = 0
IA32 = 1
PPC32 = 2
X64 = 3
ARM = 4
ValidButUnsupported = 5
PPC64 = 6
ARM64 = 7
MIPS32 = 8
MIPS64 = 9



_MODULE_AUXDATAENTRY = _descriptor.Descriptor(
  name='AuxDataEntry',
  full_name='gtirb.proto.Module.AuxDataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='gtirb.proto.Module.AuxDataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='gtirb.proto.Module.AuxDataEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=527,
  serialized_end=595,
)

_MODULE = _descriptor.Descriptor(
  name='Module',
  full_name='gtirb.proto.Module',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='gtirb.proto.Module.uuid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='binary_path', full_name='gtirb.proto.Module.binary_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='preferred_addr', full_name='gtirb.proto.Module.preferred_addr', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rebase_delta', full_name='gtirb.proto.Module.rebase_delta', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='file_format', full_name='gtirb.proto.Module.file_format', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='isa', full_name='gtirb.proto.Module.isa', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='gtirb.proto.Module.name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbols', full_name='gtirb.proto.Module.symbols', index=7,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='proxies', full_name='gtirb.proto.Module.proxies', index=8,
      number=16, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sections', full_name='gtirb.proto.Module.sections', index=9,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aux_data', full_name='gtirb.proto.Module.aux_data', index=10,
      number=17, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='entry_point', full_name='gtirb.proto.Module.entry_point', index=11,
      number=18, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_MODULE_AUXDATAENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=152,
  serialized_end=705,
)

_MODULE_AUXDATAENTRY.fields_by_name['value'].message_type = gtirb_dot_proto_dot_AuxData__pb2._AUXDATA
_MODULE_AUXDATAENTRY.containing_type = _MODULE
_MODULE.fields_by_name['file_format'].enum_type = _FILEFORMAT
_MODULE.fields_by_name['isa'].enum_type = _ISA
_MODULE.fields_by_name['symbols'].message_type = gtirb_dot_proto_dot_Symbol__pb2._SYMBOL
_MODULE.fields_by_name['proxies'].message_type = gtirb_dot_proto_dot_ProxyBlock__pb2._PROXYBLOCK
_MODULE.fields_by_name['sections'].message_type = gtirb_dot_proto_dot_Section__pb2._SECTION
_MODULE.fields_by_name['aux_data'].message_type = _MODULE_AUXDATAENTRY
DESCRIPTOR.message_types_by_name['Module'] = _MODULE
DESCRIPTOR.enum_types_by_name['FileFormat'] = _FILEFORMAT
DESCRIPTOR.enum_types_by_name['ISA'] = _ISA

Module = _reflection.GeneratedProtocolMessageType('Module', (_message.Message,), dict(

  AuxDataEntry = _reflection.GeneratedProtocolMessageType('AuxDataEntry', (_message.Message,), dict(
    DESCRIPTOR = _MODULE_AUXDATAENTRY,
    __module__ = 'gtirb.proto.Module_pb2'
    # @@protoc_insertion_point(class_scope:gtirb.proto.Module.AuxDataEntry)
    ))
  ,
  DESCRIPTOR = _MODULE,
  __module__ = 'gtirb.proto.Module_pb2'
  # @@protoc_insertion_point(class_scope:gtirb.proto.Module)
  ))
_sym_db.RegisterMessage(Module)
_sym_db.RegisterMessage(Module.AuxDataEntry)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\032com.grammatech.gtirb.proto'))
_MODULE_AUXDATAENTRY.has_options = True
_MODULE_AUXDATAENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
